// aHouse.cpp : implementation file
//

#include "stdafx.h"
#include "village.h"
#include "aHouse.h"

// dimensions of my house
#define  L 12.0f
#define  W 6.0f
#define  H 3.0f
#define  R 5.0f

// position relative to local origo
#define posx 7.0f
#define posy 7.0f

// helper function
void DrawBox(GLfloat dx,GLfloat dy,GLfloat dz,GLint mode)
{
	// draws the sides of a cube with sides
	// dx,dy,dz, origo in center
	// mode is GL_POLYGON or GL_LINES
	dx=dx/2.0f;
	dy=dy/2.0f;
	dz=dz/2.0f;

	glBegin(mode);//f1: front
		glNormal3f(-1.0f,0.0f,0.0f);
		glVertex3f(-dx,-dy,-dz);
		glVertex3f(-dx,-dy,dz);
		glVertex3f(dx,-dy,dz);
		glVertex3f(dx,-dy,-dz);
	glEnd();
	glBegin(mode);//f2: bottom
		glNormal3f(0.0f,0.0f,-1.0f);
		glVertex3f(-dx,-dy,-dz);
		glVertex3f(dx,-dy,-dz);
		glVertex3f(dx,dy,-dz);
		glVertex3f(-dx,dy,-dz);
	glEnd();
	glBegin(mode);// f3:back
		glNormal3f(1.0f,0.0f,0.0f);
		glVertex3f(dx,dy,-dz);
		glVertex3f(dx,dy,dz);
		glVertex3f(-dx,dy,dz);
		glVertex3f(-dx,dy,-dz);
	glEnd();
	glBegin(mode);// f4: top
		glNormal3f(0.0f,0.0f,1.0f);
		glVertex3f(dx,dy,dz);
		glVertex3f(dx,-dy,dz);
		glVertex3f(-dx,-dy,dz);
		glVertex3f(-dx,dy,dz);
	glEnd();
	glBegin(mode);//f5: left
		glNormal3f(0.0f,1.0f,0.0f);
		glVertex3f(-dx,-dy,-dz);
		glVertex3f(-dx,dy,-dz);
		glVertex3f(-dx,dy,dz);
		glVertex3f(-dx,-dy,dz);
	glEnd();
	glBegin(mode);//f6: right
		glNormal3f(0.0f,-1.0f,0.0f);
		glVertex3f(dx,-dy,-dz);
		glVertex3f(dx,-dy,dz);
		glVertex3f(dx,dy,dz);
		glVertex3f(dx,dy,-dz);
	glEnd();
}


IMPLEMENT_DYNCREATE(aHouse, house)

aHouse::aHouse(GLfloat x,GLfloat y)
	: house(x,y)
{
}

aHouse::aHouse()
:house()
{
}


void aHouse::DrawOutline()
{
	// make a fast outline as a wireframe box
	//------------------------------
	// Remember origo, so we can set it 
	// back when we finish drawing
	glPushMatrix();
	// set origo in areas corner
	glTranslatef(origo_X,origo_Y,0.0f);

	//------------------------------
	glTranslatef(posx+W/2.0f,posy+L/2.0f,0.0f);

	// setMaterial kind of red
	CMaterial::SetMaterial(RED_PLASTIC);
	DrawBox(W,L,R,GL_LINES);


	glPopMatrix();
}
void aHouse::Draw()
{
	// Draws a little red house
	//------------------------------
	// Remember origo, so we can set it 
	// back when we finish drawing
	glPushMatrix();
	// set origo in areas corner
	glTranslatef(origo_X,origo_Y,0.0f);

	//------------------------------
	// prepare material 
	
	// for walls
	GLfloat wall_amb[]={0.99f,0.17f,0.02f,1.0f};
	GLfloat wall_dif[]={0.94f,0.13f,0.01f,1.0f};
	GLfloat wall_spec[]={0.95f,0.31f,0.01f,1.0f};
	GLfloat wall_shine=1.0f;

	// for roof
	GLfloat roof_amb[]={0.19f,0.19f,0.19f,1.0f};
	GLfloat roof_dif[]={0.33f,0.35f,0.34f,1.0f};
	GLfloat roof_spec[]={0.35f,0.31f,0.43f,1.0f};
	GLfloat roof_shine=1.0f;
	
	//-----------------------
	// draw house as polygons
	//----------------------
	// move into the are
	glTranslatef(posx,posy,0.0f);
	// walls
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,wall_amb);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,wall_dif);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,wall_spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,wall_shine);

	glBegin(GL_POLYGON);// end
		glNormal3f(0.0f,-1.0f,0.0f);
		glVertex3f(0.0f,0.0f,0.0f);
		glVertex3f(W,0.0f,0.0f);
		glVertex3f(W,0.0f,H);
		glVertex3f(W/2.0f,0.0f,R);
		glVertex3f(0.0f,0.0f,H);
	glEnd();
	glBegin(GL_POLYGON);// other end
		glNormal3f(0.0f,1.0f,0.0f);
		glVertex3f(0.0f,L,0.0f);
		glVertex3f(W,L,0.0f);
		glVertex3f(W,L,H);
		glVertex3f(W/2.0f,L,R);
		glVertex3f(0.0f,L,H);
	glEnd();
	glBegin(GL_POLYGON);// side
		glNormal3f(-1.0f,0.0f,0.0f);
		glVertex3f(0.0f,0.0f,0.0f);
		glVertex3f(0.0f,L,0.0f);
		glVertex3f(0.0f,L,H);
		glVertex3f(0.0f,0.0f,H);
	glEnd();
	glBegin(GL_POLYGON);// otherside
		glNormal3f(1.0f,0.0f,0.0f);
		glVertex3f(W,0.0f,0.0f);
		glVertex3f(W,L,0.0f);
		glVertex3f(W,L,H);
		glVertex3f(W,0.0f,H);
	glEnd();

	//-----------------------
	// roof
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,roof_amb);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,roof_dif);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,roof_spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,roof_shine);

	glBegin(GL_POLYGON);// roofside
		glNormal3f(0.0f,0.0f,1.0f); // erronous normal
		glVertex3f(0.0f,0.0f,H);
		glVertex3f(0.0f,L,H);
		glVertex3f(W/2.0f,L,R);
		glVertex3f(W/2.0f,0.0f,R);
	glEnd();

	glBegin(GL_POLYGON);// other roofside
		glNormal3f(0.0f,0.0f,1.0f); // erronous normal
		glVertex3f(W,0.0f,H);
		glVertex3f(W,L,H);
		glVertex3f(W/2.0f,L,R);
		glVertex3f(W/2.0f,0.0f,R);
	glEnd();




	glPopMatrix();
}


BOOL aHouse:: PickAction()
{
	return FALSE;
}


