// myScene.cpp : implementing CMyScene
//

#include "stdafx.h"
#include "float.h"
#include "myscene.h"

#include "mainfrm.h"
#include "ahouse.h"
#include "nohouse.h"
#include "math.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// calculating radians from degrees
#define DtoR 0.01744f
// calculating degrees from radians 
#define RtoD 57.3248f


/////////////////////////////////////////////////////////////////////////////
// CMyWin

CMyScene::CMyScene(CWnd* pW):COGLWnd(pW)
{
	// Window is created and we can 
	// set up the drawing environment
	SetUpDrawingEnvironment();// see COGLWnd
	ASSERT(m_hRC !=NULL);

	// do one-time initializing
	InitializeRContext();

	// make sure we have no houses
	m_pHouse.RemoveAll();


	// set mousing around variables
	m_Vx=m_Vy=0.0f;
	m_bOutline=FALSE;

	// looking in 45degrees
	m_VLens=45.0f;


	// set up available houses text
	g_strAvailableHouses="Available housetypes:\n a";
}

CMyScene::~CMyScene()
{
}


BEGIN_MESSAGE_MAP(CMyScene, COGLWnd)
	//{{AFX_MSG_MAP(CMyScene)
	ON_WM_MOUSEMOVE()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CMyScene::DrawScene()
{
	// to test the basic functionality of the program skeleton:
	// Replace the body of this function with this single call:
	// COGLWnd::DrawScene();
	// It should display a yellow sphere
	
	//-------------------
	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//-------------------
	// set up viewing
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	GLfloat maxx=max(2*HX,m_Max_x);
	GLfloat maxy=max(2*HY,m_Max_y);
	
	// position the scene relative to default eyeposition 
	// which is eye in origo looking down negative z. 
	// Calculate distance from eye from lensangel
	GLfloat distance=(max(maxx,maxy)/2.0f)/(GLfloat)tan(DtoR*m_VLens/2.0f);
	
	// adjust distance as user zoomes by vertical mouse
	distance+=m_Vy;
	// take it down
	glTranslatef(0.0f,0.0f,-distance);
	// tilt if around x-axis to get a side view
	glRotatef(-70.0f,1.0f,0.0f,0.0f);
	// spin it around z-axis as user moves mouse horizontally
	glRotatef(m_Vx,0.0f,0.0f,1.0f);
	// move scene to center
	glTranslatef(-maxx/2.0f,-maxy/2.0f,0.0f);


	//------------------
	// draw the ground as a simple rctangular polygon
	// slightly below the xy-plane to avoid messing with
	// houses
	
	// material for the ground is some dead greenish color
	// could consider a textture here.
	// Spec copied from Mated
	GLfloat amb[]={0.31f,0.42f,0.35f,1.0f};
	GLfloat dif[]={0.41f,0.58f,0.11f,1.0f};
	GLfloat spec[]={0.0f,0.0f,0.0f,1.0f};
	GLfloat shine=0.0f;
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,amb);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,dif);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,shine);

	glBegin(GL_POLYGON);
		glNormal3f(0.0f,0.0f,1.0f);
		glVertex3f(-HX,-HY,-0.01f);
		glVertex3f(maxx+HX,-HY,-0.01f);
		glVertex3f(maxx+HX,maxy+HY,-0.01f);
		glVertex3f(-HX,maxy+HY,-0.01f);
	glEnd();

	
	//-------------------
	// draw all houses in the village
	for(int ix=0;ix<m_pHouse.GetSize();ix++)
	{
		if(m_bOutline)
			((house*)m_pHouse[ix])->DrawOutline();
		else
			((house*)m_pHouse[ix])->Draw();
	}
	
	//--------------------
	// finnish drawing
	glFlush();
}

void CMyScene::InitializeRContext()
{
	// to test the program skeleton:
	// replace the body of this function with this single call:
	// COGLWnd::InitializeRContext();
	// It should set up a single ligth source and 
	// a light gray background
	
	// Here we set the attributes that probably
	// will be constant during the rendering 
	// contexts lifetime.
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    
	// prepare a ligthsource
	GLfloat ambient[] = {0.4f,0.4f,0.4f,1.0f };
    GLfloat diffuse[] = {1.0f,1.0f,1.0f,1.0f };
    GLfloat position[] = {200.0f,300.0f,100.0f,0.0f };
    GLfloat lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };


    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	
	// smooth the drawing
	glShadeModel(GL_SMOOTH);

    // set background to light gray
	glClearColor(0.9f, 0.9f, 0.9f, 1.0f);
}


void CMyScene::OnDestroy() 
{
	COGLWnd::OnDestroy();
	EraseVillage();
}

void CMyScene::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	m_bOutline=TRUE;
}


void CMyScene::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_bOutline)
	{
		m_Vy+=0.5f*GLfloat(m_PLast.y-point.y);
		m_Vx+=GLfloat(m_PLast.x-point.x);
		m_PLast=point;
		RedrawWindow();
	}
}

void CMyScene::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	m_bOutline=FALSE;
	RedrawWindow();
}


void CMyScene::SetUpVillage(CString T)
{
	// constants:
	// HX width of the area disposable for a house
	// HY height
	// ROAD between lines an empty areas (space)
	// all defined in stdafx.h
	// erase old first
	EraseVillage();
	if(T.IsEmpty())
		T="z";
	// interpret text and generate houses
	T.Replace("\r\n","\n");
	m_Max_x=0.0f;
	m_Max_y=0.0f;
	GLfloat x=0.0f;
	GLfloat y=0.0f;
	for(int ix=0;ix<T.GetLength();ix++)
	{
		switch(T[ix])
		{
		case ' ':	x+=ROAD;									break;
		case '\n':	y+=ROAD+HY; x=0.0f;							break;
		case 'a':	m_pHouse.Add(new aHouse(x,y)); x+=ROAD+HX;	break;
		// here is where you add new classes after the pattern above


		default:	m_pHouse.Add(new NOHouse(x,y)); x+=ROAD+HX;
			
		}
		if(x>m_Max_x)
			m_Max_x=x;
		if(y>m_Max_y)
			m_Max_y=y;
	}
	RedrawWindow();
}

void CMyScene::EraseVillage()
{
	// destroy all houses
	for(int ix=0;ix<m_pHouse.GetSize();ix++)
	{
		if(m_pHouse[ix]!=NULL)
		{
			if( ((CObject*)m_pHouse[ix])->IsKindOf(RUNTIME_CLASS(aHouse)) )
				delete (aHouse*)m_pHouse[ix];
			// here is where you add new delete lines as you add new classes
			// like this:
			// else if( ((CObject*)m_pHouse[ix])->IsKindOf(RUNTIME_CLASS(bHouse)) )
			//	delete (bHouse*)m_pHouse[ix];
			else if( ((CObject*)m_pHouse[ix])->IsKindOf(RUNTIME_CLASS(NOHouse)) )
				delete (NOHouse*)m_pHouse[ix];
		}
	}
	m_pHouse.RemoveAll();
}

void CMyScene::SetSize(int cx, int cy)
{
	// use the whole clientarea
	glViewport(0, 0, cx, cy);

    // We will work on the projectiuon matrix,
	// so set matrix mode
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	// set up your viewing volume
	// gluPerspective:	pyramide or 
	gluPerspective(m_VLens, (GLdouble)cx/cy, 0.1f, 1000.0f);
	
	// Alternative: glOrtho:box, parallell projection
	//glOrtho(-HX,-HY,m_Max_x*cy/cx,m_Max_x*cy/cx,-10.f,1000.0f);
	// NOTE: want x/y-ratio to match viewports x/y-ratio
	
	// set back matrix mode
	glMatrixMode(GL_MODELVIEW);
}


